# -*- coding:utf-8 -*-
import logging


COLOR_FORMAT = (
    "%(color)s%(asctime)s [%(process)d] [%(thread)d] %(levelname)s : %(message)s%(color_stop)s"
)


class ColorFormatter(logging.Formatter):

    LEVEL_COLORS = {
        logging.DEBUG: '\033[00;32m',  # GREEN
        logging.INFO: '\033[00;36m',  # CYAN
        logging.WARN: '\033[01;33m',  # BOLD YELLOW
        logging.ERROR: '\033[01;31m',  # BOLD RED
        logging.CRITICAL: '\033[01;31m',  # BOLD RED
    }

    COLOR_STOP = '\033[0m'

    def add_color(self, record):
        if getattr(record, "_is_tty", False) and record.levelno > logging.INFO:
            record.color = self.LEVEL_COLORS[record.levelno]
            record.color_stop = self.COLOR_STOP
        else:
            record.color = ""
            record.color_stop = ""

    @staticmethod
    def remove_color(record):
        del record.color
        del record.color_stop

    def format(self, record):
        self.add_color(record)
        s = super(ColorFormatter, self).format(record)
        self.remove_color(record)
        return s


TEXT_FORMATTER = ColorFormatter(fmt=COLOR_FORMAT)


# -*- coding:utf-8 -*-
import os
import sys
import logging
import logging.handlers
from .formatter import TEXT_FORMATTER
from .handler import TTYHandler


OUTPUT_STDOUT = 1
OUTPUT_FILE = 2
OUTPUT_ROTATE_FILE = 3


def get_log_file_path(file_name=None):
    if not file_name:
        log_directory = os.path.join(os.path.dirname(sys.argv[0]), "log")
        if not os.path.exists(log_directory):
            os.mkdir(log_directory)
        log_name = ".".join(os.path.basename(sys.argv[0]).split(".")[:-1]) + "-" + str(os.getpid()) + ".log"
        file_name = os.path.join(log_directory,  log_name)
    return file_name


class Output(object):
    def __init__(self, handler, formatter=TEXT_FORMATTER, level=logging.INFO):
        self.handler = handler
        self.handler.setFormatter(formatter)
        self.handler.setLevel(level)


class Stream(Output):
    def __init__(self, stream=sys.stdout, formatter=TEXT_FORMATTER, level=logging.INFO):
        super(Stream, self).__init__(TTYHandler(stream), formatter, level)


class File(Output):
    def __init__(self, filename=None, formatter=TEXT_FORMATTER, level=logging.INFO):
        log_file_path = get_log_file_path(file_name=filename)
        handler = logging.FileHandler(filename=log_file_path)
        super(File, self).__init__(handler, formatter, level)


class RotatingFile(Output):
    def __init__(self, filename=None, formatter=TEXT_FORMATTER, level=logging.INFO):
        log_file_path = get_log_file_path(file_name=filename)
        handler = logging.handlers.RotatingFileHandler(filename=log_file_path, maxBytes=10*1024*1024, backupCount=10)
        super(RotatingFile, self).__init__(handler, formatter, level)


OUTPUT_MAPPING = {
    OUTPUT_STDOUT: "Stream()",
    OUTPUT_FILE: "File()",
    OUTPUT_ROTATE_FILE: "RotatingFile()",
}

def get_output_obj_list(outputs):
    output_list = list(map(eval, [OUTPUT_MAPPING.get(output, None) for output in outputs]))
    if output_list and all(output_list):
        return output_list
    else:
        raise ValueError


# -*- coding:utf-8 -*-
import logging


class TTYHandler(logging.StreamHandler):

    def format(self, record):
        if hasattr(self.stream, "isatty"):
            record._is_tty = self.stream.isatty()
        else:
            record._is_tty = False
        s = super(TTYHandler, self).format(record)
        del record._is_tty
        return s


# encoding: utf-8

import os
import io
import sys
from setuptools import setup, find_packages, Command
from shutil import rmtree

NAME = 'reelog'
DESCRIPTION = 'python log best practice.'
URL = ''
EMAIL = 'samrui0129@gmail.com'
AUTHOR = 'Sam Rui'
REQUIRES_PYTHON = '>=2.7.0'
VERSION = '1.7.1'

REQUIRED = [
]

EXTRAS = {
}

here = os.path.abspath(os.path.dirname(__file__))

try:
    with io.open(os.path.join(here, 'README.rst'), encoding='utf-8') as f:
        long_description = '\n' + f.read()
except FileNotFoundError:
    long_description = DESCRIPTION


class UploadCommand(Command):
    """Support setup.py upload."""

    description = 'Build and publish the package.'
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        try:
            print('Removing previous builds…')
            rmtree(os.path.join(here, 'dist'))
        except OSError:
            pass

        print('Building Source and Wheel (universal) distribution…')
        os.system('{0} setup.py sdist bdist_wheel --universal'.format(sys.executable))

        print('Uploading the package to PyPI via Twine…')
        os.system('twine upload dist/*')

        # print('Pushing git tags…')
        # os.system('git tag v{0}'.format(VERSION))
        # os.system('git push --tags')

        sys.exit()

setup(
    name=NAME,
    version=VERSION,
    description=DESCRIPTION,
    long_description=long_description,
    long_description_content_type='text/x-rst',
    author=AUTHOR,
    author_email=EMAIL,
    python_requires=REQUIRES_PYTHON,
    url=URL,
    packages=find_packages(exclude=('tests',)),
    install_requires=REQUIRED,
    extras_require=EXTRAS,
    include_package_data=True,
    zip_safe=False,
    license='Apache License',
    classifiers=[
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: Implementation :: CPython',
    ],
    cmdclass={
        'upload': UploadCommand,
    }
)



# -*- coding:utf-8 -*-
import logging
import weakref

from .output import get_output_obj_list, OUTPUT_STDOUT, OUTPUT_FILE, OUTPUT_ROTATE_FILE


_REE_LOGGERS = weakref.WeakValueDictionary()


def get_logger(name="root", stdout=True, file=False, rotate_file=False, level=logging.INFO):
    if not name:
        name = "root"
    logger = _REE_LOGGERS.get(name)
    if not logger:
        outputs = []
        if stdout is False and file is False and rotate_file is False:
            raise ValueError("reelog out type is None.")
        elif file is True and rotate_file is True:
            raise  ValueError("reelog out type file and rotate_file can not coexist")
        if stdout is True:
            outputs.append(OUTPUT_STDOUT)
        if file is True:
            outputs.append(OUTPUT_FILE)
        if rotate_file is True:
            outputs.append(OUTPUT_ROTATE_FILE)
        logger = _init_logger(name, outputs, level)
        _REE_LOGGERS[name] = logger

    return logger


def _init_logger(name, outputs, level):
    logger = logging.getLogger(name)
    logger.setLevel(level)
    [logger.removeHandler(handler) for handler in logger.handlers]
    [logger.addHandler(output_obj.handler) for output_obj in get_output_obj_list(outputs)]
    return logger
